function[Gam,Popt]=zernikebootstrp(Z,R,th)
%
%  [Gam]=zernikebootstrp(Z,R,th)
%
%  inputs:
%        Z  - column vector of height data
%        R  - column vector of radial data
%        th - column vector of angular data

% Created by D. Robert Iskander
%
% See D. R. Iskander, M. J. Collins, and B. Davis 
% Optimal modeling of corneal surfaces with Zernike polynomials
% IEEE Transactions on Biomedical Engineering, 48(1):87-95, 2001.

P=78;                      %Select the highest order
N=length(R);   
Rs=R./max(R);
Zdata=Z/max(R);             % scale to the unit circle
R=Rs; ZerP=zernike(R,th,P); %Zernike coefficients are according to Noll
a_hat=(Zdata'/ZerP')';      %least squares estimate of "a"
Zdata_hat=ZerP*a_hat;
r=Zdata-Zdata_hat;          %Compute resuduals
LD=0.025;                   %The choice of LD is to increase the variability 
                            %of the residuals. For wavefront data use larger LD
r=(r-mean(r))*sqrt(N/LD);   %Centre residuals
B=200;                      %Number of resamplings
Gam=zeros(1,P);
for p=1:P,
   disp(p) 
   Zerp=ZerP(:,1:p);
   r_star=r(ceil(N*rand(N,B)));   G=zeros(1,B);
   for ii=1:B,
      Zdata_star=Zerp*a_hat(1:p)+r_star(:,ii);
      a_hatstar=(Zdata_star'/Zerp')';    
      G(ii)=mean((Zdata-Zerp*a_hatstar).^2);
   end
   Gam(p)=mean(G);
end

[m,Popt]=min(Gam);         % The optimal model order


%%%%%%%LOCAL FUNCTION zernike(R,Theta,La)%%%%%%

function[Zernike] = zernike(R,Theta,La)

% Builds the discrete Zernike Polynomials

L=length(R); ia=1:La;
da=0.5*(1+sqrt(8*ia-7));
index_n=floor(da)-1;
h=floor((index_n+1).*(da-index_n-1));
index_m=h+mod(index_n+mod(h,2),2);
Zernike=zeros(L,La);
for ii=1:La,
   n=index_n(ii);    m=index_m(ii);
   R_nm=Rnm(n,m,R,L);   
   if m==0,
      Zernike(:,ii)=sqrt(n+1).*R_nm;
   else
      if ~rem(ii,2),
         Zernike(:,ii)=sqrt(2*(n+1)).*R_nm.*cos(m.*Theta);
      else
         Zernike(:,ii)=sqrt(2*(n+1)).*R_nm.*sin(m.*Theta);
      end
   end   
end
%%%%%%%LOCAL FUNCTION Rnm(n,m,R)%%%%%%%%
function[R_nm]=Rnm(n,m,R,L)
%
% R_nm=Rnm(n,m,R)
% This function calculates the coefficients of 
% the sums of Zernike polynomial

R_nm=zeros(L,1);
for s=0:1:(n-m)/2;
   R_nm=R_nm+(-1).^s.*prod(1:n-s)./prod(1:s)./ ...
      prod(1:(n+m)/2-s)./prod(1:(n-m)/2-s).*R.^(n-2*s);   
end