function[Gam]=zernikebootstrp2(Z,R,th,M)
%
%  [Gam]=zernikebootstrp2(Z,R,th)
%
%  inputs:
%        Z  - column vector of height data
%        R  - column vector of radial data
%        th - column vector of angular data

% Created by D. Robert Iskander
% July 2002
% 
% This one is based on the idea to sort R values into bins of M
% samples and perform bootstrap resampling of the residuals within 
% those bins
% L_D is a function of R (ranging from 0.1 at 1mm to 0.05 at 4mm pupil
% y=0.0133*x-0.0033;

% L_D is a function of R (ranging from 0.05 at 1mm to 0.01 at 4mm pupil
% y=0.0167*x+0.0333;

% Details of this algorithm can be found in
% A.M Zoubir and D. R. Iskander, "Bootstrap Techniques for Signal Processing"
% Cambridge University Press, 2004 and in
% D. R. Iskander, M. R. Morelande, M. J. Collins, and T. Buehren, "A refined bootstrap 
% method for estimating the Zernike polynomial model order for corneal surfaces", 
% IEEE Transactions on Biomedical Engineering, 51(12):2203-2206, December 2004.



P=100;                       % select the highest order
N=length(R); 
%M=128;                       % number of samples in a bin
NumBins=floor(N/M);
NewN=NumBins*M;
Z=Z(1:NewN);
R=R(1:NewN);
th=th(1:NewN);

Rmax=max(R);
ind=find(R~=0);
Rmin=min(R(ind));
Rs=R./Rmax;
Zdata=Z/Rmax;               % scale to the unit circle
R=Rs; ZerP=zernike(R,th,P);
a_hat=(Zdata'/ZerP')';      % least squares estimate of a
Zdata_hat=ZerP*a_hat;
r=Zdata-Zdata_hat;          % compute resuduals

[sortedR,ord]=sort(R);      % sort R
[ords,ord_inv]=sort(ord);   % you need this to get beck
rs=r(ord);                  % sort equivalently the residuals     
% For a matrix of residuals with each column corresponding to each bin

rsMat=reshape(rs,M,NumBins);

LDmax=0.0167*Rmax+0.0333;
LDmin=0.0167*Rmin+0.0333;

%LDmax=0.6667*Rmax;  %was 2.6667
%LDmin=0.6667*Rmin;

LD=ones(M,1)*linspace(LDmin,LDmax,NumBins);
rsMat=(rsMat-ones(M,1)*mean(rsMat)).*sqrt(M./LD);   %Centre residuals

B=200;                      %Number of resamplings 
P=30;
Gam=zeros(1,P);
for p=1:P,
   disp(p) 
   Zerp=ZerP(:,1:p);
   G=zeros(1,B);
   %r_star=r(ceil(N*rand(N,B)));   
   for ii=1:B,       
      % Create the boostrap version of rsMat
      rsMat_star=rsMat; %initiate
      for jj=1:NumBins,
          rsMat_star(:,jj)=rsMat(ceil(M*rand(M,1)),jj);
      end
      rs_star=reshape(rsMat_star,NewN,1); 
      rs_star=rs_star(ord_inv);                    
      Zdata_star=Zerp*a_hat(1:p)+rs_star;
      a_hatstar=(Zdata_star'/Zerp')';    
      G(ii)=mean((Zdata-Zerp*a_hatstar).^2);
   end
   Gam(p)=mean(G);
end



%%%%%%%LOCAL FUNCTION zernike(R,Theta,La)%%%%%%
function[Zernike] = zernike(R,Theta,La)

% Builds the discrete Zernike Polynomials

L=length(R); ia=1:La;
da=0.5*(1+sqrt(8*ia-7));
index_n=floor(da)-1;
h=floor((index_n+1).*(da-index_n-1));
index_m=h+mod(index_n+mod(h,2),2);
Zernike=zeros(L,La);
for ii=1:La,
   n=index_n(ii);    m=index_m(ii);
   R_nm=Rnm(n,m,R,L);   
   if m==0,
      Zernike(:,ii)=sqrt(n+1).*R_nm;
   else
      if ~rem(ii,2),
         Zernike(:,ii)=sqrt(2*(n+1)).*R_nm.*cos(m.*Theta);
      else
         Zernike(:,ii)=sqrt(2*(n+1)).*R_nm.*sin(m.*Theta);
      end
   end   
end
%%%%%%%LOCAL FUNCTION Rnm(n,m,R)%%%%%%%%
function[R_nm]=Rnm(n,m,R,L)
%
% R_nm=Rnm(n,m,R)
% This function calculates the coefficients of 
% the sums of Zernike polynomial

R_nm=zeros(L,1);
for s=0:1:(n-m)/2;
   R_nm=R_nm+(-1).^s.*prod(1:n-s)./prod(1:s)./ ...
      prod(1:(n+m)/2-s)./prod(1:(n-m)/2-s).*R.^(n-2*s);   
end